Scriptname PD_Config extends SKI_ConfigBase  

PD_DefeatHandler Property DH Auto
PD_EventManager Property EM Auto
PD_RobberyManager Property RM Auto
PD_FollowerManager Property FM Auto
PD_Util Property Util Auto
PD_RestraintsManager Property RestraintsManager Auto
PD_Rescue Property PR Auto

String[] RegEventKeys
String[] RelEventKeys

Bool Property StrugglesEnabled = true Auto Hidden
Bool Property DHLPSuspend = true Auto Hidden
Bool Property StopOnSatisfaction = true Auto Hidden
Int Property MaxRounds = 3 Auto Hidden
Int Property TimeToFlee = 15 Auto Hidden

String Property FemaleOnMaleTags = "Aggressive, Femdom" Auto Hidden
String Property MaleOnFemaleTags = "Aggressive" Auto Hidden
String Property MaleOnMaleTags = "Aggressive" Auto Hidden
String Property FemaleOnFemaleTags = "Aggressive" Auto Hidden

Bool Property FemaleOnMale = true Auto Hidden
Bool Property MaleOnFemale = true Auto Hidden
Bool Property MaleOnMale = true Auto Hidden
Bool Property FemaleOnFemale = true Auto Hidden

Float Property SatisfactionChance = 0.5 Auto Hidden

PD_AmpManager Property AmpManager Auto

Bool StruggleAPIAvailable

Quest Property RobberyQuest Auto
Message Property AbandonRobberyMsg Auto

Function Initialize()
    StruggleAPIAvailable = Game.GetModByName("AcheronExtensionLibrary.esp") != 255
    if !StruggleAPIAvailable
        StrugglesEnabled = false
    endIf
EndFunction

Function LoadPages()
	Pages = new string[5]
	Pages[0] = "General"
    Pages[1] = "Robbery"
	Pages[2] = "Events"
    Pages[3] = " Restraints"
    Pages[4] = " Miscellaneous"
EndFunction

Event OnConfigInit()
	LoadPages()
EndEvent

Event OnConfigOpen()
    LoadPages()
EndEvent

Event OnConfigClose()
    int i = 0
    while i < RegEventKeys.Length
        StorageUtil.UnsetIntValue(none, RegEventKeys[i])
        i += 1
    endWhile

    i = 0
    while i < RelEventKeys.Length
        StorageUtil.UnsetIntValue(none, RelEventKeys[i])
        i += 1
    endWhile
EndEvent

Event OnPageReset(string page)
    if page == Pages[0] || page == ""
        SetCursorFillMode(TOP_TO_BOTTOM)

        AddHeaderOption("Defeat Settings")
        int flag = OPTION_FLAG_NONE
        if !StruggleAPIAvailable
            flag = OPTION_FLAG_DISABLED
        endIf
        OID_StrugglesEnabled = AddToggleOption("Struggles Enabled", StrugglesEnabled, flag)
        OID_DHLPSuspend = AddToggleOption("DHLP Suspend", DHLPSuspend)
        OID_NumDaysRemembered = AddSliderOption("Rememberance Threshold", DH.NumDaysRemembered, "{0}d")
        OID_ShortDefeatThreshold = AddSliderOption("Quick Defeat Threshold", DH.ShortDefeatThreshold, "{0}d")
        OID_ShortDefeatSkipEvents = AddToggleOption("Quick Defeat Skip Events", DH.ShortDefeatSkipEvents)

        AddHeaderOption("Gender Preferences")
        OID_MaleOnMale = AddToggleOption("Male on Male", MaleOnMale)
        OID_FemaleOnMale = AddToggleOption("Female on Male", FemaleOnMale)
        OID_MaleOnFemale = AddToggleOption("Male on Female", MaleOnFemale)
        OID_FemaleOnFemale = AddToggleOption("Female on Female", FemaleOnFemale)

        SetCursorPosition(1)
        
        AddHeaderOption("Round Settings")
        OID_StopOnSatisfaction = AddToggleOption("Stop on Aggressor Satisfaction", StopOnSatisfaction)
        OID_SatisfactionChance = AddSliderOption("Satisfaction Chance", SatisfactionChance * 100, "{0}%")
        OID_MaxRounds = AddSliderOption("Max # of Rounds", MaxRounds)

        AddHeaderOption("Sex Tags")
        OID_MaleOnMaleTags = AddInputOption("Male on Male", MaleOnMaleTags)
        OID_FemaleOnMaleTags = AddInputOption("Female on Male", FemaleOnMaleTags)
        OID_MaleOnFemaleTags = AddInputOption("Male on Female", MaleOnFemaleTags)
        OID_FemaleOnFemaleTags = AddInputOption("Female on Female", FemaleOnFemaleTags)
        
        AddHeaderOption("Escape Options")
        OID_TimeToFlee = AddSliderOption("Time to Flee", TimeToFlee, "{0}s")
        OID_ShortDefeatRemoteReleaseOnly = AddToggleOption("Quick Defeat Remote Release", DH.ShortDefeatRemoteReleaseOnly)

    elseIf page == Pages[1]
        SetCursorFillMode(TOP_TO_BOTTOM)

        OID_RobberyEnabled = AddToggleOption("Robbery Enabled", RM.RobberyEnabled)
        OID_ShowRobbedItem = AddToggleOption("Show Robbed Item", RM.ShowRobbedItem)
        OID_AbbreviateRobberyChance = AddSliderOption("Abbreviate Robbery", RM.AbbreviateRobberyChance, "{0}%")

        AddHeaderOption("Weapons ")
        OID_RobWeaponsChance = AddSliderOption("Rob Chance", RM.RobWeaponsChance * 100, "{0}%")
        OID_RobWeaponsMinValue = AddSliderOption("Min Value", RM.RobWeaponsMinValue)

        AddHeaderOption("Armor/Clothes ")
        OID_RobClothesChance = AddSliderOption("Rob Chance", RM.RobClothesChance * 100, "{0}%")
        OID_RobClothesMinValue = AddSliderOption("Min Value", RM.RobClothesMinValue)

        AddHeaderOption("Keys ")
        OID_RobKeysChance = AddSliderOption("Rob Chance", RM.RobKeysChance * 100, "{0}%")

        SetCursorPosition(1)
        OID_FollowerRobberyEnabled = AddToggleOption("Follower Robbery Enabled", RM.FollowerRobberyEnabled)
        OID_ShortDefeatSkipRobbery = AddToggleOption("Quick Defeat Skip Robbery", DH.ShortDefeatSkipRobbery)

        int reg = OPTION_FLAG_DISABLED
        string status = "Not Running"
        if RobberyQuest.IsRunning()
            reg = OPTION_FLAG_NONE
            status = "Running"
        endIf

        OID_AbandonRobberyQuest = AddTextOption("Abandon Retrieval", status, reg)

        AddHeaderOption("Gold ")
        OID_GoldStealChance = AddSliderOption("Rob Chance", RM.GoldStealChance * 100, "{0}%")
        OID_GoldStealAmount = AddSliderOption("Amount    ", RM.GoldStealAmount * 100, "{0}%")
   
        AddHeaderOption("Other Items ")
        OID_RobItemsChance = AddSliderOption("Rob Chance", RM.RobItemsChance * 100, "{0}%")
        OID_RobItemsMinValue = AddSliderOption("Min Value", RM.RobItemsMinValue)

    elseIf page == Pages[2]
        SetCursorFillMode(TOP_TO_BOTTOM)

        
        ; OID_RegisterEvents = AddTextOption("Register Events", "")

        AddHeaderOption("Regular Events")

        RegEventKeys = PapyrusUtil.StringArray(EM.NumRegularEvents)
        MakeEventWeightSliders(RelEventKeys, EM.AllRegularEvents, EM.NumRegularEvents, "Reg")

        SetCursorPosition(1)
        ; OID_ValidateEvents = AddTextOption("Validate Events", "")

        AddHeaderOption("Release Events")

        RelEventKeys = PapyrusUtil.StringArray(EM.NumReleaseEvents)
        MakeEventWeightSliders(RelEventKeys, EM.AllReleaseEvents, EM.NumReleaseEvents, "Rel")
    elseIf page == Pages[3]
        SetCursorFillMode(TOP_TO_BOTTOM)

        AddHeaderOption("Devious Devices")
        if RestraintsManager.DDPresent
            OID_MaleFollowerRestraintsEnabled = AddToggleOption("Male Follower Restraints", RestraintsManager.MaleFollowerRestraintsEnabled)
            OID_FemaleFollowerRestraintsEnabled = AddToggleOption("Female Follower Restraints", RestraintsManager.FemaleFollowerRestraintsEnabled)

            OID_ArmRestraintsChance = AddSliderOption("Arm Chance", RestraintsManager.ArmRestraintsChance * 100, "{0}%")
            OID_LegRestraintsChance = AddSliderOption("Leg Chance", RestraintsManager.LegRestraintsChance * 100, "{0}%")
            OID_GagsChance = AddSliderOption("Gag Chance", RestraintsManager.GagsChance * 100, "{0}%")
            OID_BlindfoldsChance = AddSliderOption("Blindfold Chance", RestraintsManager.BlindfoldsChance * 100, "{0}%")
            OID_HoodsChance = AddSliderOption("Hood Chance", RestraintsManager.HoodsChance * 100, "{0}%")
            OID_APlugsChance = AddSliderOption("Anal Plug Chance", RestraintsManager.APlugsChance * 100, "{0}%")
            OID_VPlugsChance = AddSliderOption("Vaginal Plug Chance", RestraintsManager.VPlugsChance * 100, "{0}%")
            OID_CollarsChance = AddSliderOption("Collar Chance", RestraintsManager.CollarsChance * 100, "{0}%")
            OID_MiscChance = AddSliderOption("Misc Chance", RestraintsManager.MiscChance * 100, "{0}%")
            
            AddHeaderOption("Restraint Tampering")
            OID_TightenChance = AddSliderOption("Tighten", RestraintsManager.TightenChance)
        else
            AddTextOption("Devious Devices", "Not Installed")
        endIf
       
        SetCursorPosition(1)

        AddHeaderOption("Devious Devices Equip")
        if RestraintsManager.DDePresent
            OID_UseDDe = AddToggleOption("Use Devious Devices Equip", RestraintsManager.UseDDe)
            int i = 0
            while i < RestraintsManager.DDeOutfits.Length
                int res = AddInputOption("Outfit " + i, RestraintsManager.DDeOutfits[i])
                StorageUtil.SetIntValue(none, "PD_DDeOutfit_" + res, i)
                i += 1
            endWhile
        else
            AddTextOption("Devious Devices Equip", "Not Installed")
        endIf

        SetCursorPosition(1)

    elseIf page == Pages[4]
        SetCursorFillMode(TOP_TO_BOTTOM)
        AddHeaderOption("Amputation")
        if AmpManager.AmputatorPresent
            int i = 0
            while i < AmpManager.AmpFactions.Length
                string eventKey = "PD_AmpLimbChance_" + AddSliderOption(AmpManager.GetIndexLimb(i) + " Chance", AmpManager.Chances[i] * 100, "{0}%")
                StorageUtil.SetIntValue(none, eventKey, i)

                eventKey = "PD_AmpLimbLevel_" + AddSliderOption("Max Level", AmpManager.MaxLevels[i])
                StorageUtil.SetIntValue(none, eventKey, i)

                i += 1
            endWhile
        else
            AddTextOption("Amputator", "Not Installed")
        endIf

        SetCursorPosition(1)
        AddHeaderOption("Follower Slavery Mod")
        if Game.GetModByName("Follower Slavery Mod.esp") != 255
            OID_EnslaveToEnemy = AddToggleOption("Enslave to Enemy", FM.EnslaveToEnemy)
        else
            AddTextOption("Follower Slavery Mod", "Not Installed")
        endIf

        AddHeaderOption("Rescue")
        OID_EquipClothes = AddToggleOption("Equip Clothes", PR.EquipClothes)
        OID_EquipFootwear = AddToggleOption("Equip Footwear", PR.EquipFootwear)
    endIf
EndEvent

Function MakeEventWeightSliders(string[] eventKeys, PD_EventTemplate[] events, int numEvents, string prefix)
    int i = 0
    while i < numEvents
        int flag = OPTION_FLAG_NONE
        if !events[i].CanEnable
            flag = OPTION_FLAG_DISABLED
        endIf
        string eventKey = "PD_" + prefix + "EventWeightSlider_" + AddSliderOption(events[i].EventName, events[i].Weight, "{0}", flag)
        StorageUtil.SetIntValue(none, eventKey, i)
        eventKeys[i] = eventKey
        
        i += 1
    endWhile
EndFunction

Function AddDivider(string label = "")
    string divider = "----------------------------"
    divider = StringUtil.Substring(divider, 0, StringUtil.GetLength(divider) - StringUtil.GetLength(label))

    AddTextOption(divider, label)
EndFunction

Event OnOptionSelect(int a_option)
    if a_option == OID_MaleOnMale
        MaleOnMale = !MaleOnMale
        SetToggleOptionValue(OID_MaleOnMale, MaleOnMale)
    elseIf a_option == OID_MaleOnFemale
        MaleOnFemale = !MaleOnFemale
        SetToggleOptionValue(OID_MaleOnFemale, MaleOnFemale)
    elseIf a_option == OID_FemaleOnFemale
        FemaleOnFemale = !FemaleOnFemale
        SetToggleOptionValue(OID_FemaleOnFemale, FemaleOnFemale)
    elseIf a_option == OID_FemaleOnMale
        FemaleOnMale = !FemaleOnMale
        SetToggleOptionValue(OID_FemaleOnMale, FemaleOnMale)
    elseIf a_option == OID_StopOnSatisfaction
        StopOnSatisfaction = !StopOnSatisfaction
        SetToggleOptionValue(OID_StopOnSatisfaction, StopOnSatisfaction)
    elseIf a_option == OID_StrugglesEnabled
        StrugglesEnabled = !StrugglesEnabled
        SetToggleOptionValue(OID_StrugglesEnabled, StrugglesEnabled)
    elseIf a_option == OID_DHLPSuspend
        DHLPSuspend = !DHLPSuspend
        SetToggleOptionValue(OID_DHLPSuspend, DHLPSuspend)
    elseIf a_option == OID_ShortDefeatSkipRobbery
        DH.ShortDefeatSkipRobbery = !DH.ShortDefeatSkipRobbery
        SetToggleOptionValue(OID_ShortDefeatSkipRobbery, DH.ShortDefeatSkipRobbery)
    elseIf a_option == OID_ShortDefeatSkipEvents
        DH.ShortDefeatSkipEvents = !DH.ShortDefeatSkipEvents
        SetToggleOptionValue(OID_ShortDefeatSkipEvents, DH.ShortDefeatSkipEvents)
    elseIf a_option == OID_ShortDefeatRemoteReleaseOnly
        DH.ShortDefeatRemoteReleaseOnly = !DH.ShortDefeatRemoteReleaseOnly
        SetToggleOptionValue(OID_ShortDefeatRemoteReleaseOnly, DH.ShortDefeatRemoteReleaseOnly)
    elseIf a_option == OID_EnslaveToEnemy
        FM.EnslaveToEnemy = !FM.EnslaveToEnemy
        SetToggleOptionValue(OID_EnslaveToEnemy, FM.EnslaveToEnemy)
    elseIf a_option == OID_EquipClothes
        PR.EquipClothes = !PR.EquipClothes
        SetToggleOptionValue(OID_EquipClothes, PR.EquipClothes)
    elseIf a_option == OID_EquipFootwear
        PR.EquipFootwear = !PR.EquipFootwear
        SetToggleOptionValue(OID_EquipFootwear, PR.EquipFootwear)
    elseIf a_option == OID_RobberyEnabled
        RM.RobberyEnabled = !RM.RobberyEnabled
        SetToggleOptionValue(OID_RobberyEnabled, RM.RobberyEnabled)
    elseIf a_option == OID_ShowRobbedItem
        RM.ShowRobbedItem = !RM.ShowRobbedItem
        SetToggleOptionValue(OID_ShowRobbedItem, RM.ShowRobbedItem)
    elseIf a_option == OID_FollowerRobberyEnabled
        RM.FollowerRobberyEnabled = !RM.FollowerRobberyEnabled
        SetToggleOptionValue(OID_RobberyEnabled, RM.FollowerRobberyEnabled)
    elseIf a_option == OID_AbandonRobberyQuest
        if AbandonRobberyMsg.Show()
            RobberyQuest.Reset()
        endIf
    elseIf a_option == OID_MaleFollowerRestraintsEnabled
        RestraintsManager.MaleFollowerRestraintsEnabled = !RestraintsManager.MaleFollowerRestraintsEnabled
        SetToggleOptionValue(OID_MaleFollowerRestraintsEnabled, RestraintsManager.MaleFollowerRestraintsEnabled)
    elseIf a_option == OID_FemaleFollowerRestraintsEnabled
        RestraintsManager.FemaleFollowerRestraintsEnabled = !RestraintsManager.FemaleFollowerRestraintsEnabled
        SetToggleOptionValue(OID_FemaleFollowerRestraintsEnabled, RestraintsManager.FemaleFollowerRestraintsEnabled)
    elseIf a_option == OID_UseDDe
        RestraintsManager.UseDDe = !RestraintsManager.UseDDe
        SetToggleOptionValue(OID_UseDDe, RestraintsManager.UseDDe)
    elseIf a_option == OID_RegisterEvents
        EM.RequestRegister()
        Debug.MessageBox("Registering Events - Close MCM and Wait 5 Seconds")
    elseIf a_option == OID_RegisterEvents
        EM.RequestValidity()
        Debug.MessageBox("Validating Events - Close MCM and Wait 5 Seconds")
    endIf
EndEvent

Event OnOptionDefault(int a_option)
    if a_option == OID_MaleOnMale
        MaleOnMale = true
        SetToggleOptionValue(OID_MaleOnMale, MaleOnMale)
    elseIf a_option == OID_MaleOnFemale
        MaleOnFemale = true
        SetToggleOptionValue(OID_MaleOnFemale, MaleOnFemale)
    elseIf a_option == OID_FemaleOnFemale
        FemaleOnFemale = true
        SetToggleOptionValue(OID_FemaleOnFemale, FemaleOnFemale)
    elseIf a_option == OID_FemaleOnMale
        FemaleOnMale = true
        SetToggleOptionValue(OID_FemaleOnMale, FemaleOnMale)
    elseIf a_option == OID_StrugglesEnabled
        StrugglesEnabled = true
        SetToggleOptionValue(OID_StrugglesEnabled, StrugglesEnabled)
    elseIf a_option == OID_DHLPSuspend
        DHLPSuspend = true
        SetToggleOptionValue(OID_DHLPSuspend, DHLPSuspend)
    elseIf a_option == OID_EnslaveToEnemy
        FM.EnslaveToEnemy = false
        SetToggleOptionValue(OID_EnslaveToEnemy, FM.EnslaveToEnemy)
    elseIf a_option == OID_EquipClothes
        PR.EquipClothes = true
        SetToggleOptionValue(OID_EquipClothes, PR.EquipClothes)
    elseIf a_option == OID_EquipFootwear
        PR.EquipFootwear = true
        SetToggleOptionValue(OID_EquipFootwear, PR.EquipFootwear)
    elseIf a_option == OID_StopOnSatisfaction
        StopOnSatisfaction = true
        SetToggleOptionValue(OID_StopOnSatisfaction, StopOnSatisfaction)
    elseIf a_option == OID_RobberyEnabled
        RM.RobberyEnabled = true
        SetToggleOptionValue(OID_RobberyEnabled, RM.RobberyEnabled)
    elseIf a_option == OID_ShowRobbedItem
        RM.ShowRobbedItem = true
        SetToggleOptionValue(OID_ShowRobbedItem, RM.ShowRobbedItem)
    elseIf a_option == OID_FollowerRobberyEnabled
        RM.FollowerRobberyEnabled = true
        SetToggleOptionValue(OID_RobberyEnabled, RM.FollowerRobberyEnabled)
    elseIf a_option == OID_FemaleFollowerRestraintsEnabled
        RestraintsManager.FemaleFollowerRestraintsEnabled = true
        SetToggleOptionValue(OID_FemaleFollowerRestraintsEnabled, RestraintsManager.FemaleFollowerRestraintsEnabled)
    elseIf a_option == OID_MaleFollowerRestraintsEnabled
        RestraintsManager.MaleFollowerRestraintsEnabled = true
        SetToggleOptionValue(OID_MaleFollowerRestraintsEnabled, RestraintsManager.MaleFollowerRestraintsEnabled)
    elseIf a_option == OID_UseDDe
        RestraintsManager.UseDDe = false
        SetToggleOptionValue(OID_UseDDe, RestraintsManager.UseDDe)
    endIf
EndEvent

Function MakeChanceSlider(float start, float default = 1.0)
    SetSliderDialogStartValue(start * 100)
    SetSliderDialogDefaultValue(default * 100)
    SetSliderDialogRange(0, 100)
    SetSliderDialogInterval(1)
EndFunction

Function MakeValueSlider(float start, int default = 0)
    SetSliderDialogStartValue(start)
    SetSliderDialogDefaultValue(default)
    SetSliderDialogRange(0, 10000)
    SetSliderDialogInterval(10)
EndFunction

event OnOptionSliderOpen(int a_option)
	if (a_option == OID_MaxRounds)
		SetSliderDialogStartValue(MaxRounds)
		SetSliderDialogDefaultValue(3)
		SetSliderDialogRange(1, 8)
		SetSliderDialogInterval(1)
    elseIf a_option == OID_TimeToFlee
        SetSliderDialogStartValue(TimeToFlee)
		SetSliderDialogDefaultValue(15)
		SetSliderDialogRange(1, 60)
		SetSliderDialogInterval(1)
    elseIf a_option == OID_NumDaysRemembered
        SetSliderDialogStartValue(DH.NumDaysRemembered)
		SetSliderDialogDefaultValue(14.0)
		SetSliderDialogRange(0, 30)
		SetSliderDialogInterval(0.25)
    elseIf a_option == OID_ShortDefeatThreshold
        SetSliderDialogStartValue(DH.ShortDefeatThreshold)
		SetSliderDialogDefaultValue(2.0)
		SetSliderDialogRange(0, 30)
		SetSliderDialogInterval(0.25)
    elseIf a_option == OID_AbbreviateRobberyChance
        MakeChanceSlider(RM.AbbreviateRobberyChance)
    elseIf a_option == OID_SatisfactionChance
        MakeChanceSlider(SatisfactionChance)
    elseIf a_option == OID_GoldStealChance
        MakeChanceSlider(RM.GoldStealChance)
    elseIf a_option == OID_GoldStealAmount
        MakeChanceSlider(RM.GoldStealAmount)
    elseIf a_option == OID_RobClothesChance
        MakeChanceSlider(RM.RobClothesChance)
    elseIf a_option == OID_RobKeysChance
        MakeChanceSlider(RM.RobKeysChance)
    elseIf a_option == OID_RobClothesMinValue
        MakeValueSlider(RM.RobClothesMinValue)
    elseIf a_option == OID_RobWeaponsChance
        MakeChanceSlider(RM.RobWeaponsChance)
    elseIf a_option == OID_RobWeaponsMinValue
        MakeValueSlider(RM.RobWeaponsMinValue)
    elseIf a_option == OID_RobItemsChance
        MakeChanceSlider(RM.RobItemsChance)
    elseIf a_option == OID_RobItemsMinValue
        MakeValueSlider(RM.RobItemsMinValue)
    elseIf a_option == OID_ArmRestraintsChance
        MakeChanceSlider(RestraintsManager.ArmRestraintsChance, 100)
    elseIf a_option == OID_LegRestraintsChance
        MakeChanceSlider(RestraintsManager.LegRestraintsChance, 100)
    elseIf a_option == OID_GagsChance
        MakeChanceSlider(RestraintsManager.GagsChance, 100)
    elseIf a_option == OID_BlindfoldsChance
        MakeChanceSlider(RestraintsManager.BlindfoldsChance, 100)
    elseIf a_option == OID_HoodsChance
        MakeChanceSlider(RestraintsManager.HoodsChance, 100)
    elseIf a_option == OID_APlugsChance
        MakeChanceSlider(RestraintsManager.APlugsChance, 100)
    elseIf a_option == OID_VPlugsChance
        MakeChanceSlider(RestraintsManager.VPlugsChance, 100)
    elseIf a_option == OID_CollarsChance
        MakeChanceSlider(RestraintsManager.CollarsChance, 100)
    elseIf a_option == OID_MiscChance
        MakeChanceSlider(RestraintsManager.MiscChance, 100)
    elseIf a_option == OID_TightenChance
        MakeChanceSlider(RestraintsManager.TightenChance, 100)
    elseIf a_option == OID_JamChance
        MakeChanceSlider(RestraintsManager.JamChance, 100)
    else
        int regIndex = StorageUtil.GetIntValue(none, "PD_RegEventWeightSlider_" + a_option, -1)
        if regIndex > -1
            SetSliderDialogStartValue(EM.AllRegularEvents[regIndex].Weight)
            SetSliderDialogDefaultValue(100)
            SetSliderDialogRange(0, 100)
            SetSliderDialogInterval(1)
        endIf

        int relIndex = StorageUtil.GetIntValue(none, "PD_RelEventWeightSlider_" + a_option, -1)
        if relIndex > -1
            SetSliderDialogStartValue(EM.AllReleaseEvents[relIndex].Weight)
            SetSliderDialogDefaultValue(100)
            SetSliderDialogRange(0, 100)
            SetSliderDialogInterval(1)
        endIf

        int ampIndex = StorageUtil.GetIntValue(none, "PD_AmpLimbChance_" + a_option, -1)
        if ampIndex > -1
            MakeChanceSlider(AmpManager.Chances[ampIndex], 100)
        endIf

        ampIndex = StorageUtil.GetIntValue(none, "PD_AmpLimbLevel_" + a_option, -1)
        if ampIndex > -1
            SetSliderDialogStartValue(AmpManager.MaxLevels[ampIndex])
            SetSliderDialogDefaultValue(2)
            SetSliderDialogRange(0, 3)
            SetSliderDialogInterval(1)
        endIf
    endIf
endEvent

event OnOptionSliderAccept(int a_option, float a_value)
    string format = "{0}%"
	if a_option == OID_MaxRounds
		MaxRounds = a_value as int
        format = "{0}"
    elseIf a_option == OID_TimeToFlee
        TimeToFlee = a_value as int
        format = "{0}s"
    elseIf a_option == OID_NumDaysRemembered
        DH.NumDaysRemembered = a_value
        format = "{0}d"
    elseIf a_option == OID_ShortDefeatThreshold
        DH.ShortDefeatThreshold = a_value
        format = "{0}d"
    elseIf a_option == OID_SatisfactionChance
        SatisfactionChance = a_value / 100
    elseIf a_option == OID_GoldStealChance
        RM.GoldStealChance = a_value / 100
    elseIf a_option == OID_GoldStealAmount
        RM.GoldStealAmount = a_value / 100
    elseIf a_option == OID_RobClothesChance
        RM.RobClothesChance = a_value / 100
    elseIf a_option == OID_AbbreviateRobberyChance
        RM.AbbreviateRobberyChance = a_value / 100
    elseIf a_option == OID_RobKeysChance
        RM.RobKeysChance = a_value / 100
    elseIf a_option == OID_RobClothesMinValue
        RM.RobClothesMinValue = a_value as int
    elseIf a_option == OID_RobWeaponsChance
        RM.RobWeaponsChance = a_value / 100
    elseIf a_option == OID_RobWeaponsMinValue
        RM.RobWeaponsMinValue = a_value as int
    elseIf a_option == OID_RobItemsChance
        RM.RobItemsChance = a_value / 100
    elseIf a_option == OID_RobItemsMinValue
        RM.RobItemsMinValue = a_value as int
    elseIf a_option == OID_ArmRestraintsChance
        RestraintsManager.ArmRestraintsChance = a_value / 100
    elseIf a_option == OID_LegRestraintsChance
        RestraintsManager.LegRestraintsChance = a_value / 100
    elseIf a_option == OID_GagsChance
        RestraintsManager.GagsChance = a_value / 100
    elseIf a_option == OID_BlindfoldsChance
        RestraintsManager.BlindfoldsChance = a_value / 100
    elseIf a_option == OID_HoodsChance
        RestraintsManager.HoodsChance = a_value / 100
    elseIf a_option == OID_APlugsChance
        RestraintsManager.APlugsChance = a_value / 100
    elseIf a_option == OID_VPlugsChance
        RestraintsManager.VPlugsChance = a_value / 100
    elseIf a_option == OID_CollarsChance
        RestraintsManager.CollarsChance = a_value / 100
    elseIf a_option == OID_MiscChance
        RestraintsManager.MiscChance = a_value / 100
    elseIf a_option == OID_TightenChance
        RestraintsManager.TightenChance = a_value / 100
    elseIf a_option == OID_JamChance
        RestraintsManager.JamChance = a_value / 100
    else
        int regIndex = StorageUtil.GetIntValue(none, "PD_RegEventWeightSlider_" + a_option, -1)
        if regIndex > -1
            EM.AllRegularEvents[regIndex].Weight = a_value as int
            format = "{0}"
        endIf

        int relIndex = StorageUtil.GetIntValue(none, "PD_RelEventWeightSlider_" + a_option, -1)
        if relIndex > -1
            EM.AllReleaseEvents[relIndex].Weight = a_value as int
            format = "{0}"
        endIf
        
        int ampIndex = StorageUtil.GetIntValue(none, "PD_AmpLimbChance_" + a_option, -1)
        if ampIndex > -1
            AmpManager.Chances[ampIndex] = a_value / 100
        endIf

        ampIndex = StorageUtil.GetIntValue(none, "PD_AmpLimbLevel_" + a_option, -1)
        if ampIndex > -1
            AmpManager.MaxLevels[ampIndex] = a_value as int
            format = "{0}"
        endIf
	endIf
    
    SetSliderOptionValue(a_option, a_value, format)
endEvent

event OnOptionInputOpen(int a_option)
    if a_option == OID_MaleOnMaleTags
        SetInputDialogStartText(MaleOnMaleTags)
    elseIf a_option == OID_MaleOnFemaleTags
        SetInputDialogStartText(MaleOnFemaleTags)
    elseIf a_option == OID_FemaleOnFemaleTags
        SetInputDialogStartText(FemaleOnFemaleTags)
    elseIf a_option == OID_FemaleOnMaleTags
        SetInputDialogStartText(FemaleOnMaleTags)
    else
        int index = StorageUtil.GetIntValue(none, "PD_DDeOutfit_" + a_option, -1)
        if index >= 0
            SetInputDialogStartText(RestraintsManager.DDeOutfits[index])
        endIf
    endIf
endEvent

event OnOptionInputAccept(int a_option, string a_value)
    if a_option == OID_MaleOnMaleTags
        MaleOnMaleTags = a_value
    elseIf a_option == OID_MaleOnFemaleTags
        MaleOnFemaleTags = a_value
    elseIf a_option == OID_FemaleOnFemaleTags
        FemaleOnFemaleTags = a_value
    elseIf a_option == OID_FemaleOnMaleTags
        FemaleOnMaleTags = a_value
    else
        int index = StorageUtil.GetIntValue(none, "PD_DDeOutfit_" + a_option, -1)
        if index >= 0
            RestraintsManager.DDeOutfits[index] = a_value
        endIf
    endIf
    
    SetInputOptionValue(a_option, a_value)
endEvent

event OnOptionHighlight(int a_option)
    if (a_option == OID_StrugglesEnabled)
		SetInfoText("Enable struggling minigame. If disabled, you will be defeated immediately.")
    elseIf(a_option == OID_DHLPSuspend)
        SetInfoText("Use DHLP Suspend to prevent other mods from interrupting during defeat")
    elseIf a_option == OID_AbbreviateRobberyChance
        SetInfoText("Set chance of short robbery scene")
    elseif (a_option == OID_EnslaveToEnemy)
		SetInfoText("If FSM is called, should the follower be enslaved to an aggressor.")
	elseif (a_option == OID_StopOnSatisfaction)
		SetInfoText("Stop once all aggressors have had a turn.")
    elseif (a_option == OID_ShowRobbedItem)
		SetInfoText("Should stolen items be displayed as a notification during robbery.")
    elseif (a_option == OID_RobberyEnabled)
        SetInfoText("Should robbery occur at all.")
    elseif (a_option == OID_FollowerRobberyEnabled)
        SetInfoText("Should followers be robbed when you are.")
    elseIf a_option == OID_AbandonRobberyQuest
        SetInfoText("Abandon the currently active retrieval quest.")
    elseif (a_option == OID_GoldStealChance)
        SetInfoText("Set chance aggressors will steal gold")
    elseif (a_option == OID_GoldStealAmount)
        SetInfoText("Set proportion of gold stolen")
    elseif (a_option == OID_RobClothesChance)
        SetInfoText("Set chance of clothes stealing")
    elseIf (a_option == OID_RobKeysChance)
        SetInfoText("Set chance of keys being stolen")
    elseif (a_option == OID_RobClothesMinValue)
        SetInfoText("Set minimum value of clothes stolen")
    elseif (a_option == OID_RobWeaponsChance)
        SetInfoText("Set chance of weapons stealing")
    elseif (a_option == OID_RobWeaponsMinValue)
        SetInfoText("Set minimum value of weapon stolen")
    elseif (a_option == OID_RobItemsChance)
        SetInfoText("Set chance of miscellaneous items stealing")
    elseIf (a_option == OID_RobItemsMinValue)
        SetInfoText("Set minimum value of miscellaneous items stole")
    elseIf StorageUtil.GetIntValue(none, "PD_RegEventWeightSlider_" + a_option, -1) > -1 || StorageUtil.GetIntValue(none, "PD_RelEventWeightSlider_" + a_option, -1) > -1
        SetInfoText("Set relative weight of this event occurring.")
	endIf
endEvent

; General Page
int OID_StrugglesEnabled
int OID_DHLPSuspend


int OID_MaleOnMaleTags
int OID_MaleOnFemaleTags
int OID_FemaleOnMaleTags
int OID_FemaleOnFemaleTags

int OID_MaleOnMale
int OID_MaleOnFemale
int OID_FemaleOnMale
int OID_FemaleOnFemale

Int OID_MaxRounds

Int OID_NumDaysRemembered
Int OID_ShortDefeatThreshold
Int OID_ShortDefeatSkipRobbery
Int OID_ShortDefeatSkipEvents
Int OID_ShortDefeatRemoteReleaseOnly

Int OID_TimeToFlee

; Robbery Page
Int OID_RobberyEnabled
Int OID_FollowerRobberyEnabled
Int OID_ShowRobbedItem
Int OID_AbandonRobberyQuest
Int OID_AbbreviateRobberyChance

Int OID_GoldStealChance
Int OID_GoldStealAmount

Int OID_RobClothesChance
Int OID_RobClothesMinValue

Int OID_RobWeaponsChance
Int OID_RobWeaponsMinValue

Int OID_RobItemsChance
Int OID_RobItemsMinValue

Int OID_RobKeysChance

; Events Page
Int OID_RegisterEvents
Int OID_ValidateEvents


Int OID_StopOnSatisfaction
Int OID_SatisfactionChance

; Restraints Page
Int OID_MaleFollowerRestraintsEnabled
Int OID_FemaleFollowerRestraintsEnabled

Int OID_UseDDe

Int OID_ArmRestraintsChance
Int OID_GagsChance
Int OID_BlindfoldsChance
Int OID_LegRestraintsChance
Int OID_HoodsChance
Int OID_APlugsChance
Int OID_VPlugsChance
Int OID_CollarsChance
Int OID_MiscChance

Int OID_JamChance
Int OID_TightenChance

; Miscellaneous Page
Int OID_EnslaveToEnemy

Int OID_EquipClothes
Int OID_EquipFootwear