Scriptname PD_DefeatHandler extends Quest Conditional

PD_Util Property Util Auto
PD_EventLoop Property EventLoop Auto
PD_EventManager Property EM Auto
PD_RestraintsManager Property RM Auto

Scene Property ApproachScene Auto
Scene Property RobberyScene Auto

ReferenceAlias[] Property AggressorAliases Auto
ReferenceAlias[] Property VictimAliases Auto

Actor Property PlayerRef Auto

Actor[] Property Victims Auto Hidden
Actor[] Property Aggressors Auto Hidden

Int Property NumMaleVictims Auto Hidden
Int Property NumFemaleVictims Auto Hidden

Int Property NumMaleAggressors Auto Hidden
Int Property NumFemaleAggressors Auto Hidden

Int Property NumSatisfiableAggressors Auto Hidden
Actor[] Property SatisfiedAggressors Auto Hidden

AELStruggle struggle_api

PD_Config Property Config Auto
PD_EventTemplate LastChosen

Bool DHLPSuspended = false

Bool Property Remembers = false Auto Conditional Hidden
Bool Property ShortDefeat = false Auto Conditional Hidden
Bool Property SkipRobbery = false Auto Conditional Hidden
Bool Property SkipEvents = false Auto Conditional Hidden
Bool Property RemoteReleaseOnly = false Auto Conditional Hidden

Bool spinlock = false

String LAST_DEFEATED_TIME_KEY = "PD_LastDefeatedTime"
String LAST_DEFEATED_OUTCOME_KEY = "PD_LastDefeatedOutcome"

Float Property NumDaysRemembered = 14.0 Auto Hidden
Float Property ShortDefeatThreshold = 2.0 Auto Hidden
Bool Property ShortDefeatSkipRobbery = true Auto Hidden
Bool Property ShortDefeatSkipEvents = true Auto Hidden
Bool Property ShortDefeatRemoteReleaseOnly = true Auto Hidden

Function Setup()
    RegisterForSingleUpdate(0.7)
EndFunction

Event OnUpdate()
    SetStage(1)
EndEvent

Function StartDefeat()
    Util.LogInfo("Starting up defeat")

    if Config.DHLPSuspend
        Util.LogInfo("Suspending DHLP")
        DHLPSuspended = true
        SendModEvent("dhlp-Suspend")
    endIf

    RM.TrackRestraints(PlayerRef)

    if EventLoop.IsRunning()
        EventLoop.Stop()
    endIf

    if !Config.StrugglesEnabled || RM.IsWearingRestraints(PlayerRef)
        Acheron.RescueActor(PlayerRef, false)
        SetStage(10) ; start up robbery immediately
    else
        ApproachScene.Start() ; start up struggle
    endIf
        
    spinlock = true

    Victims = PapyrusUtil.ActorArray(VictimAliases.Length)

    i = 0
    j = 0
    while i < VictimAliases.Length
        Actor victim = VictimAliases[i].GetRef() as Actor

        if victim
            
            if victim != PlayerRef
                Acheron.DefeatActor(victim)
            endIf

            Victims[j] = victim

            if Util.GetGender(victim)
                NumFemaleVictims += 1
            else
                NumMaleVictims += 1
            endIf

            RM.TrackRestraints(victim)

            j += 1
        endIf

        i += 1
    endWhile
    
    Victims = PapyrusUtil.ResizeActorArray(Victims, j)

    Util.LogInfo("Victims: " + Victims)
    
    Aggressors = PapyrusUtil.ActorArray(AggressorAliases.Length)

    float daysPassed = Utility.GetCurrentGameTime()
    Remembers = false
    ShortDefeat = false
    SkipRobbery = false
    SkipEvents = false
    RemoteReleaseOnly = false

    int i = 0
    int j = 0
    while i < AggressorAliases.Length
        Actor aggressor = AggressorAliases[i].GetRef() as Actor

        if aggressor
            Acheron.PacifyActor(aggressor)
            Acheron.RescueActor(aggressor, false)
            Aggressors[j] = aggressor
            j += 1

            if Util.GetGender(aggressor)
                NumFemaleAggressors += 1
            else
                NumMaleAggressors += 1
            endIf

            float lastSeen = StorageUtil.GetFloatValue(aggressor, LAST_DEFEATED_TIME_KEY, daysPassed - 14)

            float diff = daysPassed - lastSeen

            if diff < NumDaysRemembered
                Remembers = true
            endIf

            if diff < ShortDefeatThreshold && ShortDefeatThreshold > 0
                ShortDefeat = true
            endIf
        endIf

        i += 1
    endWhile

    if ShortDefeat && ShortDefeatSkipRobbery
        SkipRobbery = true
    endIf

    if ShortDefeat && ShortDefeatSkipEvents
        SkipEvents = true
    endIf

    if ShortDefeat && ShortDefeatRemoteReleaseOnly
        RemoteReleaseOnly = true
    endIf
    
    Util.LogInfo("Remembers = " + Remembers + " ShortDefeat = " + ShortDefeat + " SkipRobbery = " + SkipRobbery + " SkipEvents = " + SkipEvents)

    Aggressors = PapyrusUtil.ResizeActorArray(Aggressors, j)
    Util.LogInfo("Aggressors: " + Aggressors)

    NumSatisfiableAggressors = 0
    if (Config.FemaleOnMale && NumMaleVictims > 0) || (Config.FemaleOnFemale && NumFemaleVictims > 0)
        NumSatisfiableAggressors += NumFemaleAggressors
    endIf

    if (Config.MaleOnMale && NumMaleVictims > 0) || (Config.MaleOnFemale && NumFemaleVictims > 0)
        NumSatisfiableAggressors += NumMaleAggressors
    endIf
    SatisfiedAggressors = PapyrusUtil.ActorArray(0)

    EventLoop.Start() ; just initialize the event loop (queue, etc.)

    spinlock = false
EndFunction

Function MaybeSatisfyActors(Actor[] akAggressors)
    Actor[] NewlySatisfiedActors = new Actor[128]
    int j = 0

    int i = 0
    while i < akAggressors.length
        bool satisfied = Utility.RandomFloat(0, 1.0) <= Config.SatisfactionChance
        if satisfied
            NewlySatisfiedActors[j] = akAggressors[i]
            j += 1
        endIf
        i += 1
    endWhile

    NewlySatisfiedActors = PapyrusUtil.ResizeActorArray(NewlySatisfiedActors, j)
    SatisfiedAggressors = PapyrusUtil.MergeActorArray(SatisfiedAggressors, NewlySatisfiedActors, true)
EndFunction

Bool Function AllSatisfied()
    return SatisfiedAggressors.Length == NumSatisfiableAggressors && Config.StopOnSatisfaction
EndFunction

Function StartStruggle(Actor akAggressor, Actor akVictim)
    if Config.StrugglesEnabled
        struggle_api = AELStruggle.Get()
        Util.LogInfo("Starting struggle")
        string callback = "PD_StruggleCallback" + akVictim.GetActorBase().GetName()
        Acheron.RescueActor(akVictim as Actor, false)
        If struggle_api.MakeStruggle(akAggressor, akVictim, callback, 70.0, 0.0)
            RegisterForModEvent(callback, "OnStruggleEnd")
        Else
            Util.LogErr("There was an error starting my struggle scene")
        EndIf
    else
        Util.LogInfo("Skipping struggle")
        SetStage(10)
    endIf
EndFunction

Event OnStruggleEnd(Form akVictim, Form akAggressor, bool abVictimEscaped)
    if abVictimEscaped
        Util.LogInfo("Victim was not defeated")
        EarlyRelease()
    else
        Util.LogInfo("Victim was defeated")
        SetStage(10)
    endIf
EndEvent

Function EarlyRelease()
    Debug.Notification("Enemies are hostile again.")
    LastChosen = none
    CleanFlags()

    ClearVictims()
    ClearAggressors()
    Reset()
EndFunction

Function StartRobbery()
    Util.LogInfo("Starting robbery")
    
    while spinlock
        Util.LogInfo("Spinlock preventing intro from starting")
        Utility.Wait(0.5)
    endWhile

    RobberyScene.Start()
EndFunction

Function StartEventLoop()
    Util.LogInfo("Starting event loop")
    
    if SkipEvents
        Util.LogInfo("Skipping events and going straight to release")
        EventLoop.Reset()
        StartRelease()
        return
    endIf

    int handle = ModEvent.Create("CC_AllowHostile")

    if handle
        ModEvent.PushBool(handle, true)
        ModEvent.Send(handle)
    endIf

    EventLoop.SetStage(1) ; start the event loop for real
EndFunction

Function StartRelease()
    Util.LogInfo("Starting release event")

    LastChosen = EM.SelectReleaseEvent(RemoteReleaseOnly)

    if !LastChosen.Start()
        EarlyRelease()
    endIf

    CleanFlags()

EndFunction 

Function CleanFlags()
    if DHLPSuspended
        Util.LogInfo("Clearing DHLP Suspend")
        DHLPSuspended = false
        SendModEvent("dhlp-Resume")
    endIf

    SendModEvent("CC_Revert")
EndFunction 

Function ClearAggressors()
    int i = 0
    Util.LogInfo("Clearing aggressors")
    float gameDaysPassed = Utility.GetCurrentGameTime()
    while i < Aggressors.Length
        StorageUtil.SetFloatValue(Aggressors[i], LAST_DEFEATED_TIME_KEY, gameDaysPassed)
        if LastChosen
            StorageUtil.SetStringValue(Aggressors[i], LAST_DEFEATED_OUTCOME_KEY, LastChosen.EventName)
        endIf
        Acheron.ReleaseActor(Aggressors[i])
        i += 1
    endWhile
EndFunction

Function ClearVictims()
    int i = 0
    Util.LogInfo("Clearing victims")
    while i < Victims.Length
        RM.UnrestrainActor(Victims[i])
        RM.UntrackRestraints(Victims[i])

        if Acheron.IsDefeated(Victims[i])
            Acheron.RescueActor(Victims[i], true)
        else
            Acheron.ReleaseActor(Victims[i])
        endIf
        i += 1
    endWhile
EndFunction

Function UnequipWeapons(Actor akVictim)
    UnequipHand(akVictim, 0)
    UnequipHand(akVictim, 1)
EndFunction

Function UnequipHand(Actor akVictim, int hand)
    int itemType = akVictim.GetEquippedItemType(hand)
    
    Form item
    if itemType > 0 && (itemType < 9 || itemType > 10)
        item = akVictim.GetEquippedWeapon(!hand)
    elseIf itemType == 10
        item = akVictim.GetEquippedWeapon(!hand)
    endIf

    if item
        akVictim.UnequipItem(item, false, true)
    endIf
EndFunction

Function StripVictim(Actor akVictim)
    Acheron.StripActor(akVictim)
EndFunction

Function Recover(Actor akVictim)
    Acheron.RescueActor(akVictim, false)
EndFunction