Scriptname PD_EventLoop extends Quest Conditional

PD_Util Property Util Auto
PD_EventManager Property EM Auto
PD_DefeatHandler Property DH Auto
PD_Config Property PC Auto

Int RoundsLeft
int numActiveEvents

Alias[] CurrEventQueue
Alias[] NextEventQueue

Bool Property InEvent = false Auto Hidden Conditional

Function Setup()
    Util.LogInfo("Setting up Event loop")
    RoundsLeft = PC.MaxRounds
    EM.Aggressors = DH.Aggressors
    InEvent = false

    numActiveEvents = 0
    PrepNextRound()

    if GetStage() > 0
        Util.LogInfo("Starting late")
        StartOrEndNextRound()
    endIf
EndFunction

Function PrepNextRound()
    RegisterForSingleUpdate(0.1)
EndFunction

Event OnUpdate()
    Util.LogInfo("Prepping next round")
    NextEventQueue = EM.BuildEventQueue(DH.Victims, DH.NumMaleAggressors, DH.NumFemaleAggressors)
EndEvent

Function StartOrEndNextRound()
    Util.LogInfo("Start or End Next Round " + DH.AllSatisfied() as string + " " + RoundsLeft as string + " " + NextEventQueue.Length as string)
    InEvent = false
    if (DH.AllSatisfied() && RoundsLeft != PC.MaxRounds) || RoundsLeft <= 0 || NextEventQueue.Length == 0
        Util.LogInfo("Stopping loop")
        ResetQueue(NextEventQueue) ; clear in case premature exit
        RoundsLeft = 0
        Stop()
        DH.SetStage(500)
    else
        Util.LogInfo("Starting round " + RoundsLeft)
        CurrEventQueue = NextEventQueue
        Util.LogInfo("Current Event Queue: " + CurrEventQueue)
        int i = 0
        while i < DH.Victims.Length
            PD_EventTemplate EventForVictim = CurrEventQueue[i] as PD_EventTemplate
            Util.LogInfo("Starting event " + EventForVictim.EventName)
            if EventForVictim.Start()
                Util.LogInfo("Succeeded event " + EventForVictim.EventName)
                InEvent = true
                numActiveEvents += 1
            else
                Util.LogInfo("Failed event " + EventForVictim.EventName)
            endIf
            i += 1
        endWhile

        ResetQueue(CurrEventQueue)
       
        RoundsLeft -= 1

        if RoundsLeft >= 1
            PrepNextRound()
        endIf

        if numActiveEvents == 0
            Util.LogInfo("No events started - terminating round")
            StartOrEndNextRound()
        endIf
    endIf
EndFunction

Function ResetQueue(Alias[] eventQueue)
    int i = 0
    while i < eventQueue.length
        if eventQueue[i]
            (eventQueue[i] as PD_EventTemplate).Reset()
        endIf
        i += 1
    endWhile
EndFunction

Function OnFinish()
    numActiveEvents -= 1
    Util.LogInfo("Finishing event - now active: " + numActiveEvents)
    if numActiveEvents <= 0 && IsRunning()
        StartOrEndNextRound()
    endIf
EndFunction