Scriptname PD_EventManager extends Quest

PD_Util Property Util Auto
PD_DefeatHandler Property DH Auto

PD_EventTemplate[] Property AllRegularEvents Auto Hidden
Int Property NumRegularEvents Auto Hidden

PD_EventTemplate[] Property ValidRegularEvents Auto Hidden
Int Property NumValidRegularEvents Auto Hidden

PD_EventTemplate[] Property AllReleaseEvents Auto Hidden
Int Property NumReleaseEvents Auto Hidden

PD_EventTemplate[] Property ValidReleaseEvents Auto Hidden
Int Property NumValidReleaseEvents Auto Hidden

Actor[] Property Aggressors Auto

Event OnInit()
    RegisterForSingleUpdate(2.0)
EndEvent

Event OnUpdate()
    RequestRegister()
EndEvent

Function RequestRegister()
    Util.LogInfo("Requesting events")

    AllRegularEvents = new PD_EventTemplate[100]
    NumRegularEvents = 0

    AllReleaseEvents = new PD_EventTemplate[100]
    NumReleaseEvents = 0

    InitValidEvents()

    SendModEvent("pDefeat_RequestRegister")
EndFunction

Function RequestValidity()
    InitValidEvents()
    SendModEvent("pDefeat_RequestValidity")
EndFunction

Function InitValidEvents()
    ValidRegularEvents = new PD_EventTemplate[100]
    NumValidRegularEvents = 0

    ValidReleaseEvents = new PD_EventTemplate[100]
    NumValidReleaseEvents = 0
EndFunction

Function TakeNextSlot(PD_EventTemplate newEvent)
    Util.LogInfo("Registering event " + newEvent.EventName, 0)
    if newEvent.EventType == 0
        if newEvent.Default && NumRegularEvents > 0
            PD_EventTemplate tmp = AllRegularEvents[0]
            AllRegularEvents[0] = newEvent
            newEvent = tmp            
        endIf

        AllRegularEvents[NumRegularEvents] = newEvent
        NumRegularEvents += 1
    else
        if newEvent.Default && NumReleaseEvents > 0
            PD_EventTemplate tmp = AllReleaseEvents[0]
            AllReleaseEvents[0] = newEvent
            newEvent = tmp
        endIf

        AllReleaseEvents[NumReleaseEvents] = newEvent
        NumReleaseEvents += 1
    endIf
EndFunction

Function TakeNextValidSlot(PD_EventTemplate newEvent)
    Util.LogInfo("Registering validated event " + newEvent.EventName, 0)
    if newEvent.EventType == 0
        if newEvent.Default && NumValidRegularEvents > 0
            PD_EventTemplate tmp = ValidRegularEvents[0]
            ValidRegularEvents[0] = newEvent
            newEvent = tmp            
        endIf

        ValidRegularEvents[NumValidRegularEvents] = newEvent
        NumValidRegularEvents += 1
    else
        if newEvent.Default && NumValidReleaseEvents > 0
            PD_EventTemplate tmp = ValidReleaseEvents[0]
            ValidReleaseEvents[0] = newEvent
            newEvent = tmp            
        endIf

        ValidReleaseEvents[NumValidReleaseEvents] = newEvent
        NumValidReleaseEvents += 1
    endIf
EndFunction

Alias[] Function BuildEventQueue(Actor[] victims, int numMaleAggs, int numFemaleAggs)
    Alias[] EventQueue = PapyrusUtil.AliasArray(victims.Length)
    int i = 0
    while i < victims.Length

        Util.LogInfo("Male left: " + numMaleAggs + " Female left: " + numFemaleAggs)
        PD_EventTemplate chosenEvent = SelectWeightedRandomEvent(false, victims, victims[i], numMaleAggs, numFemaleAggs)
        Util.LogInfo("Chosen Event for " + victims[i] + ": " + chosenEvent.EventName)
        Actor[] eventAggressors = chosenEvent.Select(i, Victims, Aggressors)

        int j = 0
        while j < eventAggressors.Length
            if Util.GetGender(eventAggressors[j]) == 0
                numMaleAggs -= 1
            else
                numFemaleAggs -= 1
            endIf
            j += 1
        endWhile
        Util.LogInfo("Existing Aggressors: " + Aggressors)

        Util.LogInfo("Event Aggressors: " + eventAggressors)

        Actor[] remainingAggs = GetRemainingActors(Aggressors, eventAggressors)

        Util.LogInfo("Remaining Aggressors: " + remainingAggs)

        Aggressors = PapyrusUtil.MergeActorArray(remainingAggs, eventAggressors)

        Util.LogInfo("New Aggressors: " + Aggressors)

        DH.MaybeSatisfyActors(eventAggressors)
        EventQueue[i] = chosenEvent
        i += 1
    endWhile

    return EventQueue
EndFunction

Actor[] Function GetRemainingActors(Actor[] aggs, Actor[] eventAggs)
    Actor[] newAggs

    if PapyrusUtil.GetVersion() < 40
        newAggs = PapyrusUtil.ActorArray(aggs.length)

        int i = 0
        while i < eventAggs.length
            StorageUtil.SetIntValue(eventAggs[i], "PD_ChosenAgg", 1)
            i += 1
        endWhile

        i = 0
        int j = 0
        while i < aggs.length
            if !StorageUtil.GetIntValue(aggs[i], "PD_ChosenAgg")
                newAggs[j] = aggs[i]
                j += 1
            endIf

            i += 1
        endWhile

        i = 0
        while i < eventAggs.length
            StorageUtil.UnsetIntValue(eventAggs[i], "PD_ChosenAgg")
            i += 1
        endWhile

        newAggs = PapyrusUtil.ResizeActorArray(newAggs, j)
    else
        newAggs = PapyrusUtil.GetDiffActor(aggs, eventAggs)
    endIf


    return newAggs
EndFunction

PD_EventTemplate Function SelectWeightedRandomEvent(bool release, Actor[] victims, Actor akVictim, int numMaleAggs, int numFemaleAggs, bool remoteOnly = false)
    Util.LogInfo("------- START Selecting random weighted event START ------- ")
    PD_EventTemplate[] validEvents
    int numEvents

    if release
        validEvents = ValidReleaseEvents
        numEvents = NumValidReleaseEvents
    else
        validEvents = ValidRegularEvents
        numEvents = NumValidRegularEvents
    endIf

    int i = 0

    int totalWeight = 0
    PD_EventTemplate[] selectableEvents = new PD_EventTemplate[128]
    int numSelectable = 0

    while i < numEvents

        PD_EventTemplate currEvent = validEvents[i]

        bool isValid
        if release
            isValid = currEvent.CanSelectRelease(victims, numMaleAggs, numFemaleAggs)
        else
            isValid = currEvent.CanSelect(Util.GetGender(akVictim), numMaleAggs, numFemaleAggs)
        endIf

        if remoteOnly && currEvent.Local
            isValid = false
        endIf    

        Util.LogInfo(currEvent.EventName + " Valid = " + isValid + " Weight = " + currEvent.Weight)

        if isValid && currEvent.Weight > 0
            totalWeight += currEvent.Weight
            selectableEvents[numSelectable] = currEvent
            numSelectable += 1
        endIf
        i += 1
    endWhile

    int chosenWeight = Utility.RandomInt(0, totalWeight)
    int currWeight = 0

    i = 0
    while i < numSelectable
        currWeight += selectableEvents[i].Weight

        if currWeight >= chosenWeight
            return selectableEvents[i]
        endIf
        i += 1
    endWhile

    Util.LogInfo("------- END Selecting random weighted event END ------- ")

    return validEvents[0]
EndFunction

PD_EventTemplate Function SelectReleaseEvent(bool remoteOnly = false)
    PD_EventTemplate selectedRelease = SelectWeightedRandomEvent(true, DH.Victims, DH.Victims[0], DH.NumMaleAggressors, DH.NumFemaleAggressors, remoteOnly)
    selectedRelease.Select(0, DH.Victims, DH.Aggressors)
    return selectedRelease
EndFunction