Scriptname PD_EventTemplate extends Alias  

PD_EventManager Property EM Auto Hidden
PD_EventLoop Property EL Auto Hidden
PD_Util Property Util Auto Hidden
PD_DefeatHandler Property DH Auto Hidden
PD_RestraintsManager Property RM Auto Hidden
PD_Config Property Config Auto Hidden

Bool Property Default Auto Hidden

String Property EventName Auto
Int Property EventType Auto
Int Property AggressorsHandled = 0 Auto

Bool Property CanEnable Auto Hidden
Int Property Weight = 100 Auto
Bool Property Local = false Auto
Bool Property RegisteredOnce = false Auto Hidden

Int Property BackupWeight Auto Hidden

; Internal
Event OnInit()
	RegisterForSingleUpdate(0.8)
EndEvent

Event OnUpdate()
	EM = Quest.GetQuest("PD_Main") as PD_EventManager
	Util = Quest.GetQuest("PD_Main") as PD_Util
	EL = Quest.GetQuest("PD_EventLoop") as PD_EventLoop
	DH = Quest.GetQuest("PD_DefeatHandler") as PD_DefeatHandler
	Config = Quest.GetQuest("PD_Config") as PD_Config
	RM = Quest.GetQuest("PD_Main") as PD_RestraintsManager
	BackupWeight = Weight

	OnModuleInit()
	Initialize()
EndEvent

event OnPlayerLoadGame()
	Initialize()
	OnModulePlayerLoad()
EndEvent

Function Initialize()
	RegisterForModEvent("pDefeat_RequestRegister", "OnRequestRegister")
	RegisterForModEvent("pDefeat_RequestValidity", "OnRequestValidity")	
EndFunction

Event OnRequestRegister(string eventName, string strArg, float numArg, Form sender)
	RegisteredOnce = true

	EM.TakeNextSlot(self)

	CanEnable = IsValid()

	if CanEnable
		EM.TakeNextValidSlot(self)
	endIf
EndEvent

Event OnRequestValidity(string eventName, string strArg, float numArg, Form sender)
	CanEnable = IsValid()

	if !RegisteredOnce
		EM.TakeNextSlot(self)
		RegisteredOnce = true
	endIf

	if CanEnable
		EM.TakeNextValidSlot(self)
	
	endIf
EndEvent

Function FinishEvent()
	Util.LogInfo("Finishing event " + EventName + " " + EventType)

	OnFinish()
	
	if El.IsRunning()
		EL.OnFinish()
	else
		Util.LogInfo("Stopping defeat handler")
		DH.ClearAggressors()
    	DH.ClearVictims()
        DH.Reset()
	endIf
EndFunction

; Helpers
Function BlackFade()
	Util.BlackFade(true)
EndFunction

Function RemoveBlackFade()
	Util.BlackFade(false)
EndFunction

; User implementation

Function OnModuleInit()
	{any one time initialization work}
EndFunction

Function OnModulePlayerLoad()
	{any initialization that must be performed every time}
ENdFunction

Bool Function IsValid()
	{check if this module is valid on load e.g. soft dep checking}
	return true
EndFunction

Bool Function CanSelect(int victimGender, int male, int female)
	{are the conditions required to start this regular event satisfied}
	return true
EndFunction

Bool Function CanSelectRelease(Actor[] victims, int male, int female)
	{are the conditions required to start this release event satisfied}
	return true
EndFunction

Actor[] Function Select(int victimIndex, Actor[] victims, Actor[] aggressors)
	{prep the needed victims and aggressors required to start this event and return the selected aggressors}
	return PapyrusUtil.ActorArray(0)
EndFunction

Bool Function Start()
	{actually start the event and return true or false depending on starting status}
	return true
EndFunction

Function Reset()
	{reset the event after a round has been started - useful if you maintain state across different functions}
EndFunction

Function OnFinish()
	{extra code to execute when the event is completed}
EndFunction