Scriptname PD_RobberyManager extends Quest Conditional

PD_Util Property Util Auto

Bool Property RobberyEnabled = true Auto Hidden
Bool Property FollowerRobberyEnabled = true Auto Hidden
Bool Property ShowRobbedItem = true Auto Hidden

Float Property GoldStealChance = 1.0 Auto Hidden
Float Property GoldStealAmount = 1.0 Auto Hidden

Float Property RobClothesChance = 1.0 Auto Hidden
Int Property RobClothesMinValue = 0 Auto Hidden

Float Property RobWeaponsChance = 1.0 Auto Hidden
Int Property RobWeaponsMinValue = 0 Auto Hidden

Float Property RobKeysChance = 0.3 Auto Hidden

Float Property RobItemsChance = 0.3 Auto Hidden
Int Property RobItemsMinValue = 100 Auto Hidden

Bool Property ShouldRobGold = true Auto Conditional Hidden
Bool Property ShouldRobKeys = true Auto Conditional Hidden
Bool Property ShouldRobItems = true Auto Conditional Hidden

Keyword[] Property ArmorClothingKeywords Auto
Keyword[] Property WeaponKeywords Auto
Keyword[] Property ItemKeywords Auto

MiscObject Property GoldBase Auto
Actor Property PlayerRef Auto

Quest Property RobberyQuest Auto

PD_RestraintsManager Property RM Auto

Float Property AbbreviateRobberyChance = 0.0 Auto Hidden
Bool Property AbbreviateRobbery Auto Conditional Hidden 

Function InitializeChances(Actor akTarget)
    ShouldRobItems = Utility.RandomFloat(0, 1.0) <= RobItemsChance
    if RM.HasKeys(akTarget)
        ShouldRobKeys = Utility.RandomFloat(0, 1.0) <= RobKeysChance
    else
        ShouldRobKeys = false
    endIf
    ShouldRobGold = Utility.RandomFloat(0, 1.0) <= GoldStealChance
    AbbreviateRobbery = Utility.RandomFloat(0, 1.0) <= AbbreviateRobberyChance
EndFunction 


Function RobAll(Actor akTarget, Actor akAggressor = none)
    Util.LogInfo("Stealing everything from " + akAggressor)
    RobClothes(akTarget, akAggressor)
    RobWeapons(akTarget, akAggressor)
    RobItems(akTarget, akAggressor)
    RobGold(akTarget, akAggressor)
    RobKeys(akTarget, akAggressor)
endFunction

Function RobClothes(Actor akVictim, Actor akAggressor = none)
    Util.LogInfo("Stealing clothes/armor " + akAggressor)
    Rob(akVictim, akAggressor, RobClothesChance, ArmorClothingKeywords, RobClothesMinValue) 
EndFunction

Function RobWeapons(Actor akVictim, Actor akAggressor = none)
    Util.LogInfo("Stealing weapons " + akAggressor)
    Rob(akVictim, akAggressor, RobWeaponsChance, WeaponKeywords, RobWeaponsMinValue) 
EndFunction

Function RobItems(Actor akVictim, Actor akAggressor = none)
    Util.LogInfo("Stealing items " + akAggressor)
    Rob(akVictim, akAggressor, RobItemsChance, ItemKeywords, RobItemsMinValue, ShouldRobItems) 
EndFunction

Function RobGold(Actor akVictim, Actor akAggressor = none)
    if !ShouldRobGold
        return
    endIf

    int currGoldAmt = akVictim.GetItemCount(GoldBase)
    Util.LogInfo("Stealing gold " + (currGoldAmt * GoldStealAmount))

    akVictim.RemoveItem(GoldBase, (currGoldAmt * GoldStealAmount) as int, !ShowRobbedItem, akAggressor)
EndFunction

Function RobKeys(Actor akVictim, Actor akAggressor)
    Util.LogInfo("Robbing keys from" + akVictim)
    if ShouldRobKeys
        Rob(akVictim, akAggressor, RobKeysChance, RM.KeyKeywords, 0, true)
    endIf
EndFunction

Function Rob(Actor akVictim, Actor akAggressor, Float robChance, Keyword[] keywords, Int minValue, bool bypass = false)
    if !RobberyEnabled || RobberyQuest.IsRunning()
        return
    endIf

    if akVictim != PlayerRef && !FollowerRobberyEnabled
        return
    endIf

    if !bypass && Utility.RandomFloat(0, 1.0) > robChance
        return
    endIf

    int i = 0
    Int iFormIndex = akVictim.GetNumItems()
	While iFormIndex > 0
		iFormIndex -= 1

		Form kForm = akVictim.GetNthForm(iFormIndex)
        
        int j = 0
        while j < keywords.length
            if kForm.HasKeyword(keywords[j])
                akVictim.RemoveItem(kForm, 1, !ShowRobbedItem, akAggressor)
                j = keywords.length
            endIf
            j += 1
        endWhile
	EndWhile
EndFunction

Function StartRobberyQuest(Actor Robber)
    bool result = RobberyQuest.Start()
    Util.LogInfo("Starting robbery quest = " + result)
EndFunction

Function SwapRobber()
    
EndFunction