Scriptname PD_Util extends Quest

PD_Config Property Config Auto
PD_RobberyManager Property RM Auto
SexLabFramework Property SL Auto

Actor Property PlayerRef Auto
ImageSpaceModifier Property Woozy Auto
Idle Property Wakeup Auto

Function LogInfo(string msg, int level = 1)
    Log(msg, "info", level)
EndFunction

Function LogErr(string msg, int level = 2)
    Log(msg, "error", level)
EndFunction

Function Log(string msg, string type, int level)
    msg = "PD: (" + type + ") " + msg
    Debug.Trace(msg)

    if level > 0
        Acheron.PrintConsole(msg)
    endIf

    if level > 1
        Debug.Notification(msg)
    endIf
EndFunction

int Function StartSexWithAnimations(Actor[] positions, Actor victim, sslBaseAnimation[] anims, string hook)
    sslThreadModel Thread = SL.NewThread()
	if !Thread
		LogErr("StartSex() - Failed to claim an available thread")
		return -1
    elseIf !Thread.AddActors(positions, victim)
		LogErr("StartSex() - Failed to add some actors to thread")
		return -1
	endIf
	Thread.SetAnimations(anims)
	Thread.DisableBedUse(true)
	Thread.SetHook(Hook)

    if Thread.StartThread()
		return Thread.tid
	endIf
	return -1
EndFunction

sslBaseAnimation[] Function GetAnimations(int actorCount, Actor vic, Actor agg)
    int vicGender = GetGender(vic)
    int aggGender = GetGender(agg)

    string tags = ""
    if vicGender == 0 && aggGender == 0
        tags = Config.MaleOnMaleTags
    elseIf vicGender == 0 && aggGender == 1
        tags = Config.FemaleOnMaleTags
    elseIf vicGender == 1 && aggGender == 0
        tags = Config.MaleOnFemaleTags
    else
        tags = Config.FemaleOnFemaleTags
    endIf

    sslBaseAnimation[] anims = SL.GetAnimationsByTags(actorCount, tags)

    if anims
        return anims
    else
        return SL.GetAnimationsByTags(actorCount, "")
    endIf
EndFunction

Int Function GetGender(Actor akActor)
    if !akActor
        return -1
    endIf

    return SL.GetGender(akActor)
EndFunction

Bool Function CanPair(Actor akVictim, Actor akAggressor)
    int vGender = SL.GetGender(akVictim)
    int aGender = SL.GetGender(akAggressor)

    if vGender == 0 && aGender == 0 ; male on male
        return Config.MaleOnMale
    elseIf vGender == 0 && aGender == 1 ; female on male
        return Config.FemaleOnMale
    elseIf vGender == 1 && aGender == 0 ; male on female
        return Config.MaleOnFemale
    elseIf vGender == 1 && aGender == 1 ; female on female
        return Config.FemaleOnFemale
    else
        LogErr("Creatures unsupported")
        return false
    endif
EndFunction

; 0 = none exists, 1 = male pairing, 2 = female pairing, 3 = both possible
Int Function CanCreateScene(int victimGender, int male, int female, int numAggs)

    int validAggs = 0
    bool maleValid = false
    bool femaleValid = false

    if victimGender == 0
        if Config.MaleOnMale && numAggs <= male
            maleValid = true
        endIf
        if Config.FemaleOnMale && numAggs <= female
            femaleValid = true
        endIf
    else
        if Config.MaleOnFemale && numAggs <= male
            maleValid = true
        endIf
        if Config.FemaleOnFemale && numAggs <= female
            femaleValid = true
        endIf
    endIf
    
    if !maleValid && !femaleValid
        return 0
    endIf


    int mCount
    int fCount

    if victimGender == 0
        if maleValid && male >= numAggs
            mCount = SL.GetAnimationsByType(numAggs + 1, numAggs + 1).Length
        endIf
        if femaleValid && female >= numAggs
            fCount = SL.GetAnimationsByType(numAggs + 1, 1, numAggs).Length
        endIf
    else
        if maleValid && male >= numAggs
            mCount = SL.GetAnimationsByType(numAggs + 1, numAggs, 1).Length
        endIf
        if femaleValid && female >= numAggs
            fCount = SL.GetAnimationsByType(numAggs + 1, 0, numAggs + 1).Length
        endIf
    endIf

    if fCount > 0 && mCount > 0
        return 3
    elseIf fCount > 0
        return 2
    elseIf mCount > 0
        return 1
    else
        return 0
    endIf
EndFunction

Bool Function AnimsExist(int male, int female)
    return SL.GetAnimationsByType(male + female, male, female).Length > 0
EndFunction

Function BlackFade(bool fadeOut = true)
	if FadeOut
		Game.FadeOutGame(false, true, 60.0, 0.0)
	else
		Game.FadeOutGame(false, true, 0.2, 3.0)
	endIf
EndFunction

Bool Function HasClothes(Actor akTarget)
    Int iFormIndex = akTarget.GetNumItems()
    Keyword[] keywords = RM.ArmorClothingKeywords

	While iFormIndex > 0
		iFormIndex -= 1

		Form kForm = akTarget.GetNthForm(iFormIndex)
        
        int j = 0
        while j < keywords.length
            if kForm.HasKeyword(keywords[j])
                return true
            endIf
            j += 1
        endWhile
	EndWhile

    return false
EndFunction

bool Function IsInFirstPerson()
	int cstate = game.GetCameraState()
	return (cstate == 0) ;|| ((cstate == 9))
EndFunction

Function SetSkyUIWidgetsVisible(bool visible)
	float val
	if visible
		val = 100.0
	else 
		val = 0.0
	endif 

	UI.SetNumber("HUD Menu", "_root.WidgetContainer._alpha", val)
EndFunction

Function SetUIVisible(bool visible)
	UI.SetBool("HUD Menu", "_root.HUDMovieBaseInstance._visible", visible)
EndFunction

Function WakeupWoozy()
    Woozy.Apply()
    PlayerRef.PlayIdle(WakeUp)
EndFunction

Actor[] Function GetValidAggressors(Actor[] aggressors, int numAggs, int genderOptions)
    Actor[] selectedAggs
    Actor[] malePicked = PapyrusUtil.ActorArray(numAggs)
    Actor[] femalePicked = PapyrusUtil.ActorArray(numAggs)

    int i = 0
    int j = 0
    int k = 0
    while i < aggressors.length
        if GetGender(aggressors[i]) == 0 && (genderOptions == 3 || genderOptions == 1)
            malePicked[j] = aggressors[i]
            j += 1

            if j == numAggs
                selectedAggs = malePicked
                i = aggressors.length
            endIf
        elseIf GetGender(aggressors[i]) == 1 && (genderOptions == 3 || genderOptions == 2)
            femalePicked[k] = aggressors[i] 
            k += 1

            if k == numAggs
                selectedAggs = femalePicked
                i = aggressors.length
            endIf
        endIf

        i += 1
    endWhile

    return selectedAggs
EndFunction

bool Function IsRuntimeGenerated(Form akForm)
    return akForm.GetFormID() <= -16777216
EndFunction
  